
module.exports = new Class
({
	Extends: Vn.Object

	,tpvOrder: null
	,tpvStatus: null

	,check: function (callback)
	{
		this.tpvOrder = Vn.Hash.get ('tpvOrder');
		this.tpvStatus = Vn.Hash.get ('tpvStatus');

		if (this.tpvStatus)
		{
			var batch = new Sql.Batch ();
			batch.addValue ('transaction', this.tpvOrder);
			batch.addValue ('status', this.tpvStatus);
		
			var query = 'CALL transaction_end (#transaction, #status)';
			this.conn.execQuery (query, null, batch);
		}
		
		if (callback)
			callback (this, this.tpvOrder, this.tpvStatus);
	}

	,pay: function (amount, company)
	{
		this._realPpay (amount * 100, company);
	}

	,_realPpay: function (amount, company)
	{	
		if (amount > 0)
		{
			var params = {
				 'amount': parseInt (amount)
				,'urlOk': this._makeUrl ('ok')
				,'urlKo': this._makeUrl ('ko')
				,'company': company
			};
		
			this.conn.send ('tpv/transaction', params,
				this._onTransactionStart.bind (this));
		}
		else if (!isNaN (amount))
			Htk.Toast.showError (_('AmountError'));
	}

	,_onTransactionStart: function (json, error)
	{
		if (json)
		{
			var form = document.createElement ('form');
			form.method = 'post';
			form.action = json.url;
			document.body.appendChild (form);
			
			var fieldsMap =
			{
				 'Ds_SignatureVersion': 'HMAC_SHA256_V1'
				,'Ds_MerchantParameters': json.params
				,'Ds_Signature': json.signature
			};

			for (var field in fieldsMap)
			{
				var input = document.createElement ('input');
				input.type = 'hidden';
				input.name = field;
				form.appendChild (input);
				
				if (fieldsMap[field])			
					input.value = fieldsMap[field];
			}
			
			form.submit ();
		}
		else
			Htk.Toast.showWarning (_('PayError'));
	}
	
	,retryPay: function ()
	{
		var batch = new Sql.Batch ();
		batch.addValue ('transaction', parseInt (this.tpvOrder));

		var query = 'SELECT t.amount, m.company_id '
			+'FROM tpv_transaction_view t '
			+'JOIN tpv_merchant m ON t.merchant_id = m.id '
				+'WHERE t.id = #transaction';
		this.conn.execQuery (query,
			this._onRetryPayDone.bind (this), batch);
	}
	
	,_onRetryPayDone: function (resultSet)
	{
		var res = resultSet.fetchResult ();
		
		if (res.next ())
			this._realPpay (res.get ('amount'), res.get ('company_id'));
		else
			Htk.Toast.showError (_('AmountError'));
	}

	,_makeUrl: function (status, order)
	{
		var path = location.protocol +'//'+ location.host;
		path += location.port ? ':'+ location.port : '';
		path += location.pathname;
		path += location.search ? location.search : '';
		path += Vn.Hash.make ({
			'form': 'ecomerce/orders',
			'tpvStatus': status,
			'tpvOrder': '%s'
		}, true);

		return path;
	}
});

